#!/usr/bin/env scheme-script
;; -*- mode: scheme; coding: utf-8 -*- !#
;; SPDX-License-Identifier: MIT
;; Copyright © 2020 Göran Weinholt
#!r6rs

(import
  (rnrs)
  (srfi :64)
  (d-bus protocol wire))

(define (d-bus-marshal-message type data*)
  (call-with-bytevector-output-port
    (lambda (p) (d-bus-put-message p 0 1 type data*))))

(define (roundtrip type data*)
  (let ((bv (d-bus-marshal-message type data*)))
    (let-values ([(offset decoded) (d-bus-unmarshal-message bv 0 (native-endianness) type)])
      (cond ((and (equal? decoded data*) (= (bytevector-length bv) offset))
             #t)
            (else
             (display "type: ")
             (write type)
             (newline)
             (display "data: ")
             (write data*)
             (newline)
             (display "encoded: ")
             (write bv)
             (newline)
             (display "encoded length: ")
             (write (bytevector-length bv))
             (newline)
             (display "decoded: ")
             (write decoded)
             (newline)
             (display "decode offset: ")
             (write offset)
             (newline)
             #f)))))

(test-begin "marshal")

(test-assert (roundtrip '(message BYTE BOOLEAN INT16 STRING (ARRAY INT32))
                        '(1 #t -1 "hej" #(1 2))))

(test-assert (roundtrip '(message BYTE SIGNATURE)
                        '(42 "o")))

(test-assert (roundtrip '(message SIGNATURE)
                        '("o")))

(test-assert (roundtrip '(message (STRUCT BYTE VARIANT))
                        '((3 (BYTE . #xff)))))

(test-assert (roundtrip '(message BYTE VARIANT)
                        '(3 (BYTE . #xff))))


(test-assert (roundtrip '(message (ARRAY (STRUCT INT16 INT16)))
                        '(#((1 2) (2 3)))))

(test-assert (roundtrip '(message BYTE BOOLEAN INT16 SIGNATURE STRING (ARRAY INT32))
                        '(1 #t -1 "o" "XXXXXXXXXXXX" #(1 2))))

(test-assert (roundtrip '(message (ARRAY (DICT_ENTRY BYTE STRING)))
                        '(#((1 . "a") (2 . "b")))))

(test-assert (roundtrip '(message (STRUCT BYTE INT16) STRING (ARRAY (DICT_ENTRY BYTE BYTE)))
                        '((1 2) "wer" #((1 . 1) ))))

(test-assert (roundtrip '(message INT16 SIGNATURE STRING VARIANT)
                        '(-1 "o" "XXXXXXXXX" (INT32 . 2))))

(test-end)

(exit (if (zero? (test-runner-fail-count (test-runner-get))) 0 1))
